from django.utils.translation import ugettext_lazy as _

from rest_framework import serializers

from .utils import is_bitcoin_address_valid


class BitcoinAddressField(serializers.CharField):
    default_error_messages = {
        'invalid': _('Invalid bitcoin address.')
    }

    def to_internal_value(self, data):
        data = super().to_internal_value(data)
        if not is_bitcoin_address_valid(data):
            self.fail('invalid')
        return data



from django.db import models
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext_lazy as _

from .utils import is_bitcoin_address_valid


def validate_address(value):
    if not is_bitcoin_address_valid(value):
        raise ValidationError(_('Not a valid bitcoin address.'))


class BitcoinAddressField(models.CharField):
    description = 'A bitcoin address'

    def __init__(self, *args, **kwargs):
        kwargs['max_length'] = 36
        super().__init__(*args, **kwargs)
        self.validators.append(validate_address)

    def deconstruct(self):
        name, path, args, kwargs = super().deconstruct()
        del kwargs["max_length"]
        return name, path, args, kwargs



from hashlib import sha256


digits58 = '123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz'


def decode_base58(bc, length):
    n = 0
    for char in bc:
        n = n * 58 + digits58.index(char)
    return n.to_bytes(length, 'big')


def is_bitcoin_address_valid(address):
    bcbytes = decode_base58(address, 25)
    return bcbytes[-4:] == sha256(sha256(bcbytes[:-4]).digest()).digest()[:4]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import re

from setuptools import setup


def get_version(package):
    """
    Return package version as listed in `__version__` in `init.py`.
    """
    init_py = open(os.path.join(package, '__init__.py')).read()
    return re.search("__version__ = ['\"]([^'\"]+)['\"]", init_py).group(1)


version = get_version('djbitcoin')
readme = open('README.rst').read()

setup(
    name='dj-bitcoin',
    version=version,
    url='https://github.com/silverlogic/dj-bitcoin',
    license='MIT',
    description='Django helpers for working with bitcoin',
    long_description=readme,
    author='Ryan Pineo',
    author_email='ryanpineo@gmail.com',
    packages=['djbitcoin'],
    install_requires=[],
    zip_safe=False,
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Internet :: WWW/HTTP',
    ]
)
